using System;
using System.Collections;
using System.ComponentModel;
using System.Drawing;
using System.Data;
using System.Windows.Forms;
using System.Runtime.InteropServices;

namespace ClientControls
{
	/// <summary>
	/// Summary description for Minimap.
	/// </summary>
	public class Minimap : System.Windows.Forms.PictureBox
	{
		[DllImport(@".\Unitsync.dll")]
		private static extern IntPtr GetMinimap(string filename, int miplevel);

		// image cannot be set from outside
		new protected Image Image;

		private bool			m_Loaded;
		private IntPtr			m_Memory;

		public delegate void delImageChanged(object sender);

		public event delImageChanged ImageChanged;

		/// <summary> 
		/// Required designer variable.
		/// </summary>
		private System.ComponentModel.Container components = null;

		public Minimap()
		{
			// This call is required by the Windows.Forms Form Designer.
			InitializeComponent();

			// TODO: Add any initialization after the InitializeComponent call
			m_Loaded = false;
		}

		/// <summary> 
		/// Clean up any resources being used.
		/// </summary>
		protected override void Dispose( bool disposing )
		{
			if( disposing )
			{
				if(components != null)
				{
					components.Dispose();
				}

				if ( m_Loaded )
					CloseImage();
			}
			base.Dispose( disposing );
		}

		public bool Open(string filename)
		{
			if ( m_Loaded )
				return false;
			try
			{
				m_Memory = GetMinimap(filename, 0);
				if ( m_Memory == (IntPtr)0)
					return false;
				Bitmap bmp = new Bitmap(1024, 1024, 2048, System.Drawing.Imaging.PixelFormat.Format16bppRgb565, m_Memory);
				// the 256x256 minimap looks like CRAP.  manual resize of the higher res one
				Bitmap newbmp = new Bitmap(bmp, this.Size);
				base.Image = newbmp;
				if ( ImageChanged != null )
					ImageChanged(this);
				/*
				this.Width = 256;
				this.Height = 256;
				*/
				m_Loaded = true;
				return true;
			}
			catch ( Exception ex )
			{
				MessageBox.Show(ex.ToString());
				return false;
			}
		}

		public bool CloseImage()
		{
			if ( m_Loaded == false )
				return true;
			try
			{
				Bitmap bmp = (Bitmap)base.Image;
				bmp.Dispose();
				base.Image = null;
				// this memory is freed by bitmap already
//				Marshal.FreeHGlobal(m_Memory);
				m_Loaded = false;
				return true;
			}
			catch ( Exception ex )
			{
				ex.ToString();
				return false;
			}
		}

		#region Component Designer generated code
		/// <summary> 
		/// Required method for Designer support - do not modify 
		/// the contents of this method with the code editor.
		/// </summary>
		private void InitializeComponent()
		{
			components = new System.ComponentModel.Container();
		}
		#endregion
	}
}
